TEMPLATE = """
# Namespace List

Here is a list of all namespaces with brief descriptions:

{% for node in nodes recursive %}
{%- if node.is_namespace %}
* **{{node.kind.value}}** [**{{node.name_short}}**]({{node.url}}) {{node.brief}}
  {%- if node.has_children %}
    {{- loop(node.children)|indent(2, true) }}
  {%- endif %}
{%- endif -%}
{%- endfor %}
"""



import subprocess


# Credits: https://stackoverflow.com/a/1630350
def lookahead(iterable):
    """Pass through all values from the given iterable, augmented by the
    information if there are more values to come after the current one
    (True), or if it is the last value (False).
    """
    # Get an iterator and pull the first value.
    it = iter(iterable)
    last = next(it)
    # Run the iterator to exhaustion (starting from the second value).
    for val in it:
        # Report the *previous* value (more to come).
        yield last, True
        last = val
    # Report the last value.
    yield last, False


def contains(a, pos, b):
    ai = pos
    bi = 0
    if len(b) > len(a) - pos:
        return False
    while bi < len(b):
        if a[ai] != b[bi]:
            return False
        ai += 1
        bi += 1
    return True


def split_safe(s: str, delim: str) -> [str]:
    tokens = []
    i = 0
    last = 0
    inside = 0
    while i < len(s):
        c = s[i]
        if i == len(s) - 1:
            tokens.append(s[last : i + 1])
        if c == '<' or c == '[' or c == '{' or c == '(':
            inside += 1
            i += 1
            continue
        if c == '>' or c == ']' or c == '}' or c == ')':
            inside -= 1
            i += 1
            continue
        if inside > 0:
            i += 1
            continue
        if contains(s, i, delim):
            tokens.append(s[last:i])
            i += 2
            last = i
        i += 1
    return tokens


def get_git_revision_hash() -> str:
    try:
        return subprocess.check_output(['git', 'rev-parse', 'HEAD']).decode('ascii').strip()
    except subprocess.CalledProcessError:
        return ''



import argparse
import os
import shlex
import shutil
import subprocess
import sys
from shutil import copytree

from doxybook.constants import DEFAULT_TEMPLATES_DIR, SUPPORTED_LANGS
from doxybook.runner import run


def parse_options():
    parser = argparse.ArgumentParser(description='Convert doxygen XML output into GitBook or Vuepress markdown output.')
    parser.add_argument(
        '-t',
        '--target',
        choices=['gitbook', 'vuepress', 'docsify', 'mkdocs', 'single-markdown'],
        help='markdown type',
        default='single-markdown',
    )
    parser.add_argument('-i', '--input', help='Path to doxygen generated xml folder')
    parser.add_argument('-o', '--output', help='Path to the destination folder')
    parser.add_argument(
        '-s',
        '--summary',
        type=str,
        help='Path to the summary file which contains a link to index.md in the folder pointed by --input (default: false)',
    )
    parser.add_argument(
        '-l',
        '--link-prefix',
        type=str,
        help='Adds a prefix to all links. You can use this to specify an absolute path if necessary. Docsify might need this. (default: "")',
        default='',
    )
    parser.add_argument(
        '-d', '--debug', type=bool, help='Debug the class hierarchy (default: false)', required=False, default=False
    )
    parser.add_argument(
        '--hints',
        type=bool,
        help='(Vuepress only) If set to true, hints will be generated for the sections note, bug, and a warning (default: true)',
        default=True,
    )
    parser.add_argument(
        '--ignoreerrors',
        type=bool,
        help='If set to true, will continue to generate Markdown files even if an error has been detected (default: false)',
        default=False,
    )
    parser.add_argument(
        '--template-dir',
        help='Would use this template dir instead of the default one if set. '
        'Suggest use "doxygen generate-templates" at first to generate the defaults template dir, '
        'and then make your own changes on it.',
    )
    parser.add_argument(
        '--template-lang',
        choices=SUPPORTED_LANGS,
        default='c',
        help='specifies your project\'s main language.',
    )
    parser.add_argument(
        '--doxygen-bin',
        default='doxygen',
        help='doxygen binary path',
    )
    parser.add_argument(
        '--doxygen-extra-args', default='', help='extra argument passed into doxygen. should be doublequoted'
    )

    action = parser.add_subparsers(dest='action')
    generate_templates = action.add_parser('generate-templates')
    generate_templates.add_argument(
        'output_dir', nargs='?', default=os.getcwd(), help=f'generate default template files. (Default: {os.getcwd()})'
    )

    args = parser.parse_args()

    return args


def main():
    args = parse_options()
    if args.action:
        if os.path.isfile(args.output_dir):
            raise Exception('The [OUTPUT_DIR] should be a directory')

        output_dir = os.path.join(args.output_dir, 'templates')
        if os.path.exists(output_dir):
            raise ValueError(f'{output_dir} folder already exists')

        copytree(DEFAULT_TEMPLATES_DIR, output_dir)
        print(f'Copied the default template files to {output_dir}')
        return

    doxygen_bin = shutil.which(args.doxygen_bin)
    if not doxygen_bin:
        raise RuntimeError(f'{args.doxygen_bin} not found in your PATH')

    doxygen_cmd = [doxygen_bin]
    doxygen_cmd.extend(shlex.split(args.doxygen_extra_args))
    print(f"Running {' '.join(doxygen_cmd)}")
    subprocess.run(doxygen_cmd, stderr=sys.stderr, stdout=sys.stdout, check=True)

    if args.input is None or args.output is None:
        raise ValueError('-i/--input and -o/--output are required')

    if args.target == 'gitbook' and args.summary and not os.path.exists(args.summary):
        raise Exception('The provided summary file does not exist!')

    run(
        input_dir=args.input,
        output=args.output,
        target=args.target,
        hints=args.hints,
        debug=args.debug,
        ignore_errors=args.ignoreerrors,
        summary=args.summary,
        link_prefix=args.link_prefix,
        template_dir=args.template_dir,
        template_lang=args.template_lang,
    )


if __name__ == '__main__':
    main()



import os
import re
import string
from collections import defaultdict
from typing import TextIO

from jinja2 import StrictUndefined, Template, Undefined
from jinja2.exceptions import TemplateError, TemplateSyntaxError

from doxybook.constants import Kind
from doxybook.node import DummyNode, Node
from doxybook.templates_python.annotated import TEMPLATE as ANNOTATED_TEMPLATE
from doxybook.templates_python.classes import TEMPLATE as CLASSES_TEMPLATE
from doxybook.templates_python.files import TEMPLATE as FILES_TEMPLATE
from doxybook.templates_python.hierarchy import TEMPLATE as HIEARARCHY_TEMPLATE
from doxybook.templates_python.index import TEMPLATE as INDEX_TEMPLATE
from doxybook.templates_python.member import TEMPLATE as MEMBER_TEMPLATE
from doxybook.templates_python.member_definition import TEMPLATE as MEMBER_DEFINITION_TEMPLATE
from doxybook.templates_python.member_table import TEMPLATE as MEMBER_TABLE_TEMPLATE
from doxybook.templates_python.modules import TEMPLATE as MODULES_TEMPLATE
from doxybook.templates_python.namespaces import TEMPLATE as NAMESPACES_TEMPLATE
from doxybook.templates_python.page import TEMPLATE as PAGE_TEMPLATE
from doxybook.templates_python.pages import TEMPLATE as PAGES_TEMPLATE
from doxybook.templates_python.programlisting import TEMPLATE as PROGRAMLISTING_TEMPLATE

LETTERS = string.ascii_lowercase + '~_@'

ADDITIONAL_FILES = {
    'Namespace List': 'namespaces.md',
    'Namespace Members': 'namespace_members.md',
    'Namespace Member Functions': 'namespace_member_functions.md',
    'Namespace Member Variables': 'namespace_member_variables.md',
    'Namespace Member Typedefs': 'namespace_member_typedefs.md',
    'Namespace Member Enumerations': 'namespace_member_enums.md',
    'Class Index': 'classes.md',
    'Class Hierarchy': 'hierarchy.md',
    'Class Members': 'class_members.md',
    'Class Member Functions': 'class_member_functions.md',
    'Class Member Variables': 'class_member_variables.md',
    'Class Member Typedefs': 'class_member_typedefs.md',
    'Class Member Enumerations': 'class_member_enums.md',
}


def generate_link(name, url) -> str:
    return '* [' + name + '](' + url + ')\n'


class Generator:
    def __init__(self, ignore_errors: bool = False, options: dict = {}):
        self.options = options

        on_undefined_class = Undefined
        if not ignore_errors:
            on_undefined_class = StrictUndefined

        try:
            self.annotated_template = Template(ANNOTATED_TEMPLATE, undefined=on_undefined_class)
            self.member_template = Template(MEMBER_TEMPLATE, undefined=on_undefined_class)
            self.member_definition_template = Template(MEMBER_DEFINITION_TEMPLATE, undefined=on_undefined_class)
            self.member_table_template = Template(MEMBER_TABLE_TEMPLATE, undefined=on_undefined_class)
            self.namespaces_template = Template(NAMESPACES_TEMPLATE, undefined=on_undefined_class)
            self.classes_template = Template(CLASSES_TEMPLATE, undefined=on_undefined_class)
            self.hiearchy_template = Template(HIEARARCHY_TEMPLATE, undefined=on_undefined_class)
            self.index_template = Template(INDEX_TEMPLATE, undefined=on_undefined_class)
            self.modules_template = Template(MODULES_TEMPLATE, undefined=on_undefined_class)
            self.files_template = Template(FILES_TEMPLATE, undefined=on_undefined_class)
            self.programlisting_template = Template(PROGRAMLISTING_TEMPLATE, undefined=on_undefined_class)
            self.page_template = Template(PAGE_TEMPLATE, undefined=on_undefined_class)
            self.pages_template = Template(PAGES_TEMPLATE, undefined=on_undefined_class)
        except TemplateSyntaxError as e:
            raise Exception(str(e) + ' at line: ' + str(e.lineno))

    def _render(self, tmpl: Template, path: str, data: dict) -> str:
        try:
            print('Generating', path)
            data.update(self.options)
            output = tmpl.render(data)

            with open(path, 'w', encoding='utf-8') as file:
                file.write(output)
        except TemplateError as e:
            raise Exception(str(e))

    def _recursive_find(self, nodes: [Node], kind: Kind):
        ret = []
        for node in nodes:
            if node.kind == kind:
                ret.append(node)
            if node.kind.is_parent():
                ret.extend(self._recursive_find(node.children, kind))
        return ret

    def _recursive_find_with_parent(self, nodes: [Node], kinds: [Kind], parent_kinds: [Kind]):
        ret = []
        for node in nodes:
            if node.kind in kinds and node.parent is not None and node.parent.kind in parent_kinds:
                ret.append(node)
            if node.kind.is_parent() or node.kind.is_dir() or node.kind.is_file():
                ret.extend(self._recursive_find_with_parent(node.children, kinds, parent_kinds))
        return ret

    def annotated(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'annotated.md')

        data = {'nodes': nodes}
        self._render(self.annotated_template, path, data)

    def programlisting(self, output_dir: str, node: [Node]):
        path = os.path.join(output_dir, node.refid + '_source.md')

        data = {'node': node}
        self._render(self.programlisting_template, path, data)

    def fileindex(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'files.md')

        data = {'nodes': nodes}
        self._render(self.files_template, path, data)

    def namespaces(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'namespaces.md')

        data = {'nodes': nodes}
        self._render(self.namespaces_template, path, data)

    def page(self, output_dir: str, node: Node):
        path = os.path.join(output_dir, node.name + '.md')

        data = {'node': node}
        self._render(self.page_template, path, data)

    def pages(self, output_dir: str, nodes: [Node]):
        for node in nodes:
            self.page(output_dir, node)

    def relatedpages(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'pages.md')

        data = {'nodes': nodes}
        self._render(self.pages_template, path, data)

    def classes(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'classes.md')

        classes = self._recursive_find(nodes, Kind.CLASS)
        classes.extend(self._recursive_find(nodes, Kind.STRUCT))
        classes.extend(self._recursive_find(nodes, Kind.INTERFACE))
        dictionary = defaultdict(list)

        for klass in classes:
            dictionary[klass.name_short[0].lower()].append(klass)

        for letter in list(dictionary):
            if len(dictionary[letter]) == 0:
                del dictionary[letter]

        data = {'dictionary': dictionary}
        self._render(self.classes_template, path, data)

    def _find_base_classes(self, nodes: [Node], derived: Node):
        ret = []
        for node in nodes:
            if isinstance(node, str):
                ret.append({'refid': node, 'derived': derived})
            elif node.kind.is_parent() and not node.kind.is_namespace():
                bases = node.base_classes
                if len(bases) == 0:
                    ret.append(node)
                else:
                    ret.extend(self._find_base_classes(bases, node))
        return ret

    def modules(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'modules.md')

        data = {'nodes': nodes}
        self._render(self.modules_template, path, data)

    def hierarchy(self, output_dir: str, nodes: [Node]):
        path = os.path.join(output_dir, 'hierarchy.md')

        classes = self._recursive_find(nodes, Kind.CLASS)
        classes.extend(self._recursive_find(nodes, Kind.STRUCT))
        classes.extend(self._recursive_find(nodes, Kind.INTERFACE))

        bases = self._find_base_classes(classes, None)
        deduplicated = {}

        for base in bases:
            if not isinstance(base, dict):
                deduplicated[base.refid] = base

        for base in bases:
            if isinstance(base, dict):
                if base['refid'] not in deduplicated:
                    deduplicated[base['refid']] = []
                deduplicated[base['refid']].append(base)

        deduplicated_arr = []
        for key, children in deduplicated.items():
            if isinstance(children, list):
                deduplicated_arr.append(DummyNode(key, list(map(lambda x: x['derived'], children)), Kind.CLASS))
            else:
                found: Node = None
                for klass in classes:
                    if klass.refid == key:
                        found = klass
                        break
                if found:
                    deduplicated_arr.append(found)

        data = {'classes': deduplicated_arr}
        self._render(self.hiearchy_template, path, data)

    def member(self, output_dir: str, node: Node):
        path = os.path.join(output_dir, node.filename)

        data = {
            'node': node,
            'member_definition_template': self.member_definition_template,
            'member_table_template': self.member_table_template,
        }
        self._render(self.member_template, path, data)

        if node.is_language or node.is_group or node.is_file or node.is_dir:
            self.members(output_dir, node.children)

    def file(self, output_dir: str, node: Node):
        path = os.path.join(output_dir, node.filename)

        data = {
            'node': node,
            'member_definition_template': self.member_definition_template,
            'member_table_template': self.member_table_template,
        }
        self._render(self.member_template, path, data)

        if node.is_file and node.has_programlisting:
            self.programlisting(output_dir, node)

        if node.is_file or node.is_dir:
            self.files(output_dir, node.children)

    def members(self, output_dir: str, nodes: [Node]):
        for node in nodes:
            if node.is_parent or node.is_group or node.is_file or node.is_dir:
                self.member(output_dir, node)

    def files(self, output_dir: str, nodes: [Node]):
        for node in nodes:
            if node.is_file or node.is_dir:
                self.file(output_dir, node)

    def index(self, output_dir: str, nodes: [Node], kind_filters: Kind, kind_parents: [Kind], title: str):
        path = os.path.join(output_dir, title.lower().replace(' ', '_') + '.md')

        found_nodes = self._recursive_find_with_parent(nodes, kind_filters, kind_parents)
        dictionary = {}

        # Populate initial dictionary
        for letter in LETTERS:
            dictionary[letter] = []

        # Sort items into the dictionary
        for found in found_nodes:
            dictionary[found.name_tokens[-1][0].lower()].append(found)

        # Delete unused letters
        for letter in list(dictionary):
            if len(dictionary[letter]) == 0:
                del dictionary[letter]

        # Sort items if they have the same name
        sorted_dictionary = {}
        for letter, items in dictionary.items():
            d = {}
            for item in items:
                # The name of the item is not yet in the dictionary
                if item.name_short not in d:
                    d[item.name_short] = [item.parent]

                # If the key is already in the dictionary,
                # make sure there are no duplicates.
                # For example an overloaded constructor or function!
                # Only allow distinct parents
                else:
                    found = False
                    for test in d[item.name_short]:
                        if test.refid == item.parent.refid:
                            found = True
                            break
                    if not found:
                        d[item.name_short].append(item.parent)

            sorted_dictionary[letter] = d

        data = {'title': title, 'dictionary': sorted_dictionary}
        self._render(self.index_template, path, data)

    def _generate_recursive(self, f: TextIO, node: Node, level: int, diff: str):
        if node.kind.is_parent():
            f.write(' ' * level + generate_link(node.kind.value + ' ' + node.name, diff + '/' + node.refid + '.md'))
            for child in node.children:
                self._generate_recursive(f, child, level + 2, diff)

    def _generate_recursive_files(self, f: TextIO, node: Node, level: int, diff: str):
        if node.kind.is_file() or node.kind.is_dir():
            f.write(' ' * level + generate_link(node.name, diff + '/' + node.refid + '.md'))
            if node.kind.is_file():
                f.write(' ' * level + generate_link(node.name + ' source', diff + '/' + node.refid + '_source.md'))
            for child in node.children:
                self._generate_recursive_files(f, child, level + 2, diff)

    def _generate_recursive_groups(self, f: TextIO, node: Node, level: int, diff: str):
        if node.kind.is_group():
            f.write(' ' * level + generate_link(node.title, diff + '/' + node.refid + '.md'))
            for child in node.children:
                self._generate_recursive_groups(f, child, level + 2, diff)

    def _generate_recursive_pages(self, f: TextIO, node: Node, level: int, diff: str):
        if node.kind.is_page():
            f.write(' ' * level + generate_link(node.title, diff + '/' + node.refid + '.md'))
            for child in node.children:
                self._generate_recursive_pages(f, child, level + 2, diff)

    def summary(self, output_dir: str, summary_file: str, nodes: [Node], modules: [Node], files: [Node], pages: [Node]):
        print('Modifying', summary_file)
        summaryDir = os.path.dirname(os.path.abspath(summary_file))
        output_path = os.path.abspath(output_dir)
        diff = output_path[len(summaryDir) + 1 :].replace('\\', '/')
        link = diff + '/index.md'

        content = []
        with open(summary_file) as f:
            content = f.readlines()

        start = None
        offset = 0
        end = None
        for i in range(0, len(content)):
            line = content[i]
            if start is None and re.search(re.escape(link), line):
                m = re.search('\\* \\[', line)
                if m is not None:
                    start = m.start()
                    start = i
                continue

            if start is not None and end is None:
                if not line.startswith(' ' * (offset + 2)):
                    end = i

        if start is None:
            print('WARNING: Could not generate summary! Unable to find \"* [...](' + link + ')\" in SUMMARY.md')
            return

        if end is None:
            end = len(content)

        with open(summary_file, 'w+') as f:
            # Write first part of the file
            for i in range(0, start + 1):
                f.write(content[i])

            f.write(' ' * (offset + 2) + generate_link('Related Pages', diff + '/' + 'pages.md'))
            for node in pages:
                self._generate_recursive_pages(f, node, offset + 4, diff)

            f.write(' ' * (offset + 2) + generate_link('Modules', diff + '/' + 'modules.md'))
            for node in modules:
                self._generate_recursive_groups(f, node, offset + 4, diff)

            f.write(' ' * (offset + 2) + generate_link('Class List', diff + '/' + 'annotated.md'))
            for node in nodes:
                self._generate_recursive(f, node, offset + 4, diff)

            for key, val in ADDITIONAL_FILES.items():
                f.write(' ' * (offset + 2) + generate_link(key, diff + '/' + val))

            f.write(' ' * (offset + 2) + generate_link('Files', diff + '/' + 'files.md'))
            for node in files:
                self._generate_recursive_files(f, node, offset + 4, diff)

            f.write(' ' * (offset + 2) + generate_link('File Variables', diff + '/' + 'variables.md'))
            f.write(' ' * (offset + 2) + generate_link('File Functions', diff + '/' + 'functions.md'))
            f.write(' ' * (offset + 2) + generate_link('File Macros', diff + '/' + 'macros.md'))

            # Write second part of the file
            for i in range(end, len(content)):
                f.write(content[i])



TEMPLATE = """
# {{title}}

{% for letter, items in dictionary.items() %}
## {{letter}}

{% for name, parents in items.items() -%}
* **{{name}}** (
{%- for parent in parents -%}[**{{parent.name_long}}**]({{parent.url}}){{ ', ' if not loop.last else '' }}{% endfor -%} )
{% endfor %}
{% endfor %}
"""



from xml.etree.ElementTree import Element as Element

from doxybook.cache import Cache
from doxybook.markdown import (
    Br,
    Code,
    Md,
    MdBlockQuote,
    MdBold,
    MdCodeBlock,
    MdHeader,
    MdHint,
    MdImage,
    MdItalic,
    MdLink,
    MdList,
    MdParagraph,
    MdRenderer,
    MdTable,
    MdTableCell,
    MdTableRow,
    Text,
)
from doxybook.utils import lookahead

SIMPLE_SECTIONS = {
    'see': 'See also:',
    'note': 'Note:',
    'bug': 'Bug:',
    'warning': 'Warning:',
    'return': 'Returns:',
    'returns': 'Returns:',
    'param': 'Parameters:',
    'templateparam': 'Template parameters:',
    'retval': 'Return value:',
    'author': 'Author:',
    'authors': 'Authors:',
    'since': 'Since:',
    'pre': 'Precondition:',
    'remark': 'Remark:',
    'copyright': 'Copyright:',
    'post': 'Postcondition:',
    'rcs': 'Rcs:',
    'attention': 'Attention:',
    'invariant': 'Invariant:',
    'exception': 'Exception:',
    'date': 'Date:',
}

SIMPLE_SECTIONS_HINTS_VUEPRESS = {'note': 'tip', 'bug': 'danger', 'warning': 'warning'}


class XmlParser:
    def __init__(self, cache: Cache, target: str = 'gitbook', hints: bool = True):
        self.target = target
        self.cache = cache
        self.hints = hints

    def anchor(self, name: str) -> str:
        return '<a name=\"' + name + '\"></a>'

    def paras_as_str(self, p: Element, italic: bool = False, plain: bool = False) -> str:
        if plain:
            return self.plain_as_str(p)
        else:
            renderer = MdRenderer()
            for m in self.paras(p, italic=italic):
                m.render(renderer, '')
            return renderer.output.strip()

    def reference_as_str(self, p: Element) -> str:
        renderer = MdRenderer()
        refid = p.get('refid')
        if refid is not None:
            m = MdLink([MdBold([Text(p.text)])], refid)
            m.render(renderer, '')
            return renderer.output.strip()
        else:
            return p.text.strip()

    def programlisting_as_str(self, p: Element) -> str:
        renderer = MdRenderer()
        for m in self.programlisting(p):
            m.render(renderer, '')
        return renderer.output

    def plain_as_str(self, p: Element) -> str:
        return ' '.join(self.plain(p)).strip()

    def plain(self, p: Element) -> [str]:
        ret = []
        if p is None:
            return ret
        if p.text:
            ret.append(p.text.strip())
        for item in p:
            ret.extend(self.plain(item))
        if p.tail:
            ret.append(p.tail.strip())
        return ret

    def programlisting(self, p: Element) -> [Md]:
        ret = []
        # programlisting
        if p.tag == 'programlisting':
            got_lang = False
            code = MdCodeBlock([])
            for codeline in p.findall('codeline'):
                line = ''
                for highlight in codeline.findall('highlight'):
                    if (
                        not got_lang
                        and len(highlight) == 0
                        and highlight.text is not None
                        and highlight.text.startswith('{')
                        and highlight.text.endswith('}')
                    ):
                        lang = highlight.text[1:-1]
                        code.set_lang(lang)
                        got_lang = True
                        continue
                    else:
                        if highlight.text is not None:
                            line += highlight.text
                        for c in highlight:
                            if c.tag == 'sp':
                                line += ' '
                            if c.text:
                                line += c.text
                            if c.tail:
                                line += c.tail
                code.append(line)
            ret.append(Text('\n'))
            ret.append(code)
        return ret

    def paras(self, p: Element, italic: bool = False) -> [Md]:
        ret = []
        if p is None:
            return ret
        if p.text:
            if italic:
                ret.append(MdItalic([Text(p.text.strip())]))
                ret.append(Text(' '))
            else:
                ret.append(Text(p.text))
        for item in p:
            # para
            if item.tag == 'para':
                ret.append(MdParagraph(self.paras(item)))
                ret.append(Text('\n'))

            # image
            elif item.tag == 'image':
                url = item.get('name')
                ret.append(MdImage(url))

            # computeroutput
            elif item.tag == 'computeroutput':
                text = []
                if item.text:
                    text.append(item.text)
                for i in item:
                    text.extend(self.plain(i))
                ret.append(Code(' '.join(text)))

            # programlisting
            elif item.tag == 'programlisting':
                ret.extend(self.programlisting(item))

            # table
            elif item.tag == 'table':
                t = MdTable()
                for row in item.findall('row'):
                    r = MdTableRow([])
                    for cell in row.findall('entry'):
                        for para in cell.findall('para'):
                            r.append(MdTableCell(self.paras(para)))
                    t.append(r)
                ret.append(t)

            # blockquote
            elif item.tag == 'blockquote':
                b = MdBlockQuote([])
                for para in item.findall('para'):
                    b.extend(self.paras(para))
                ret.append(b)

            # heading
            elif item.tag == 'heading':
                ret.append(MdHeader(int(item.get('level')), self.paras(item)))

            # orderedlist
            elif item.tag == 'orderedlist' or item.tag == 'itemizedlist':
                lst = MdList([])
                for listitem in item.findall('listitem'):
                    i = MdParagraph([])
                    for para in listitem.findall('para'):
                        i.extend(self.paras(para))
                    lst.append(i)
                ret.append(lst)

            # Reference
            elif item.tag == 'ref':
                refid = item.get('refid')
                try:
                    ref = self.cache.get(refid)
                    if self.target == 'single-markdown':
                        _t = ref.relative_link
                    else:
                        _t = ref.url
                    if italic:
                        if item.text:
                            ret.append(MdLink([MdItalic([MdBold([Text(item.text)])])], _t))
                        else:
                            ret.append(MdLink([MdItalic([MdBold([Text(ref.get_full_name())])])], _t))
                    else:
                        if item.text:
                            ret.append(MdLink([MdBold([Text(item.text)])], _t))
                        else:
                            ret.append(MdLink([MdBold([Text(ref.get_full_name())])], _t))
                except:
                    if item.text:
                        ret.append(Text(item.text))

            # sect1:
            elif item.tag == 'sect1':
                title = item.find('title').text
                ret.append(MdHeader(2, [Text(title)]))
                ret.extend(self.paras(item))

            # sect2:
            elif item.tag == 'sect2':
                title = item.find('title').text
                ret.append(MdHeader(3, [Text(title)]))
                ret.extend(self.paras(item))

            # sect3:
            elif item.tag == 'sect3':
                title = item.find('title').text
                ret.append(MdHeader(4, [Text(title)]))
                ret.extend(self.paras(item))

            # sect4:
            elif item.tag == 'sect4':
                title = item.find('title').text
                ret.append(MdHeader(5, [Text(title)]))
                ret.extend(self.paras(item))

            # sect5:
            elif item.tag == 'sect5':
                title = item.find('title').text
                ret.append(MdHeader(6, [Text(title)]))
                ret.extend(self.paras(item))

            # variablelist
            elif item.tag == 'variablelist':
                varlistentry = item.find('varlistentry')

                ret.append(MdHeader(4, self.paras(varlistentry.find('term'))))

                for listitem in item.findall('listitem'):
                    for para in listitem.findall('para'):
                        ret.append(MdParagraph(self.paras(para)))

            # parameterlist
            elif item.tag == 'parameterlist':
                parameteritems = item.findall('parameteritem')
                lst = MdList([])
                for parameteritem in parameteritems:
                    name = parameteritem.find('parameternamelist').find('parametername')
                    description = parameteritem.find('parameterdescription').findall('para')
                    par = MdParagraph([])
                    if len(name) > 0:
                        par.extend(self.paras(name))
                    else:
                        par.append(Code(name.text))
                    par.append(Text(' '))
                    for ip in description:
                        par.extend(self.paras(ip))
                    lst.append(par)
                ret.append(Br())
                ret.append(MdBold([Text(SIMPLE_SECTIONS[item.get('kind')])]))
                ret.append(Br())
                ret.append(lst)

            # simplesect
            elif item.tag == 'simplesect':
                kind = item.get('kind')
                if self.hints and self.target == 'vuepress' and kind in SIMPLE_SECTIONS_HINTS_VUEPRESS:
                    ret.append(Br())
                    children = []
                    for sp in item.findall('para'):
                        children.extend(self.paras(sp))
                        children.append(Br())
                    ret.append(MdHint(children, SIMPLE_SECTIONS_HINTS_VUEPRESS[kind], SIMPLE_SECTIONS[kind]))

                else:
                    ret.append(Br())
                    ret.append(MdBold([Text(SIMPLE_SECTIONS[kind])]))
                    if kind != 'see':
                        ret.append(Br())
                    else:
                        ret.append(Text(' '))

                    for sp, has_more in lookahead(item.findall('para')):
                        ret.extend(self.paras(sp))
                        if kind == 'see':
                            if has_more:
                                ret.append(Text(', '))
                        else:
                            ret.append(Br())

            # xrefsect
            elif item.tag == 'xrefsect':
                xreftitle = item.find('xreftitle')
                xrefdescription = item.find('xrefdescription')
                kind = xreftitle.text.lower()
                if self.hints and self.target == 'vuepress' and kind in SIMPLE_SECTIONS_HINTS_VUEPRESS:
                    children = []
                    for sp in xrefdescription.findall('para'):
                        children.extend(self.paras(sp))
                        children.append(Br())
                    ret.append(MdHint(children, SIMPLE_SECTIONS_HINTS_VUEPRESS[kind], SIMPLE_SECTIONS[kind]))
                else:
                    ret.append(Br())
                    ret.append(MdBold(self.paras(xreftitle)))
                    ret.append(Br())
                    for sp in xrefdescription.findall('para'):
                        ret.extend(self.paras(sp))
                        ret.append(Br())

            # Hard link
            elif item.tag == 'ulink':
                ret.append(MdLink(self.paras(item), item.get('url')))

            # Bold
            elif item.tag == 'bold':
                ret.append(MdBold(self.paras(item)))

            # Emphasis
            elif item.tag == 'emphasis':
                ret.append(MdItalic(self.paras(item)))

            # End of the item text
            if item.tail.strip():
                if italic:
                    ret.append(Text(' '))
                    ret.append(MdItalic([Text(item.tail.strip())]))
                else:
                    ret.append(Text(item.tail.strip()))
        return ret



TEMPLATE = """
# Class List

Here are the classes, structs, unions and interfaces with brief descriptions:

{% for node in nodes recursive %}
{%- if node.is_parent %}
* **{{node.kind.value}}** [**{{node.name_short}}**]({{node.url}}) {{node.brief}}
{%- if node.has_children %}
  {{- loop(node.children)|indent(2, true) }}
{%- endif %}
{%- endif -%}
{%- endfor %}
"""



class Cache:
    def __init__(self):
        self.cache = {}

    def add(self, key: str, value):
        self.cache[key] = value

    def get(self, key: str):
        if key in self.cache:
            return self.cache[key]
        else:
            raise IndexError('Key: ' + key + ' not found in cache!')



import re
from xml.etree.ElementTree import Element as Element

from doxybook.constants import Kind
from doxybook.markdown import escape
from doxybook.xml_parser import XmlParser


class Property:
    class Details:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            detaileddescription = self.xml.find('detaileddescription')
            if len(list(detaileddescription)) > 0:
                return self.parser.paras_as_str(detaileddescription, plain=plain)
            else:
                return ''

        def plain(self) -> str:
            return self.md(plain=True)

        def has(self) -> bool:
            detaileddescription = self.xml.find('detaileddescription')
            return len(list(detaileddescription)) > 0

    class Brief:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            briefdescription = self.xml.find('briefdescription')
            if not briefdescription:
                return ''

            paras = briefdescription.findall('para')
            if len(paras) > 0:
                text = []
                for para in paras:
                    text.append(self.parser.paras_as_str(para, italic=True, plain=plain))
                return ' '.join(text)
            else:
                return ''

        def plain(self) -> str:
            return self.md(plain=True)

        def has(self) -> bool:
            briefdescription = self.xml.find('briefdescription')
            return len(list(briefdescription)) > 0

    class Includes:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> [str]:
            return self.array(plain=False)

        def plain(self) -> [str]:
            return self.array(plain=True)

        def array(self, plain: bool = False) -> [str]:
            ret = []
            for includes in self.xml.findall('includes'):
                if plain:
                    incl = includes.text
                else:
                    incl = self.parser.reference_as_str(includes)

                if includes.get('local') == 'yes':
                    ret.append('"' + incl + '"')
                else:
                    if plain:
                        ret.append('<' + incl + '>')
                    else:
                        ret.append('&lt;' + incl + '&gt;')
            return ret

        def has(self) -> bool:
            return len(self.xml.findall('includes')) > 0

    class Type:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            para = self.xml.find('type')
            if para is not None:
                return self.parser.paras_as_str(para, plain=plain)
            else:
                return ''

        def plain(self) -> str:
            return self.md(plain=True)

        def has(self) -> bool:
            return self.xml.find('type') is not None

    class Location:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return self.plain()

        def plain(self) -> str:
            loc = self.xml.find('location')
            if loc is not None:
                return loc.get('file')
            else:
                return ''

        def has(self) -> bool:
            return self.xml.find('location') is not None

    class Params:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return ', '.join(self.array(plain=plain))

        def plain(self) -> str:
            return ', '.join(self.array(plain=True))

        def array(self, plain: bool = False) -> [str]:
            ret = []
            for param in self.xml.findall('param'):
                p = ''
                type = param.find('type')
                p = self.parser.paras_as_str(type, plain=plain)

                declname = param.find('declname')
                if declname is not None:
                    p += ' ' + self.parser.paras_as_str(declname, plain=plain)
                else:
                    defname = param.find('defname')
                    if defname is not None:
                        p += ' ' + self.parser.paras_as_str(defname, plain=plain)

                defval = param.find('defval')
                if defval is not None:
                    p += '=' + self.parser.paras_as_str(defval, plain=plain)

                ret.append(p.strip())
            return ret

        def has(self) -> bool:
            return len(self.xml.findall('param')) > 0

    class TemplateParams:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return ', '.join(self.array(plain=plain))

        def plain(self) -> str:
            return ', '.join(self.array(plain=True))

        def array(self, plain: bool = False, notype: bool = False) -> [str]:
            ret = []
            templateparamlist = self.xml.find('templateparamlist')
            if templateparamlist is not None:
                for param in templateparamlist.findall('param'):
                    if notype:
                        declname = param.find('declname')
                        if declname is None:
                            declname = param.find('type')
                        ret.append(self.parser.paras_as_str(declname, plain=plain))
                    else:
                        type = param.find('type')
                        declname = param.find('declname')
                        if declname is None:
                            declname = param.find('type')
                        ret.append(
                            self.parser.paras_as_str(type, plain=plain)
                            + ' '
                            + self.parser.paras_as_str(declname, plain=plain)
                        )
            return ret

        def has(self) -> bool:
            return self.xml.find('templateparamlist') is not None

    class CodeBlock:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return self.plain()

        def plain(self) -> str:
            return ''

        def has(self) -> bool:
            return True

    class Specifiers:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return self.plain()

        def plain(self) -> str:
            argss = self.xml.find('argsstring')
            if argss is None or argss.text is None:
                return ''

            argsstring = argss.text
            ret = []

            # Is deleted?
            if re.search('\\)\\s*=\\s*delete', argsstring):
                ret.append('= delete')

            # Is default?
            if re.search('\\)\\s*=\\s*default', argsstring):
                ret.append('= default')

            # Is noexcept
            if re.search('\\).*noexcept', argsstring):
                ret.append('noexcept')

            # Is override
            if re.search('\\).*override', argsstring):
                ret.append('override')

            # Is const?
            if self.xml.get('const') == 'yes':
                ret.append('const')

            # Is pure?
            virt = self.xml.get('virt')
            if virt == 'pure-virtual':
                ret.append('= 0')

            return ' '.join(ret)

        def has(self) -> bool:
            return self.xml.find('argsstring') is not None

    class Values:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return ', '.join(self.array(plain=plain))

        def plain(self) -> str:
            return ', '.join(self.array(plain=False))

        def array(self, plain: bool = False) -> [str]:
            ret = []
            if self.kind.is_enum():
                for enumvalue in self.xml.findall('enumvalue'):
                    p = '**' + escape(enumvalue.find('name').text) + '**'
                    initializer = enumvalue.find('initializer')
                    if initializer is not None:
                        p += ' ' + self.parser.paras_as_str(initializer, plain=plain)
                    ret.append(p)
            return ret

        def has(self) -> bool:
            if self.kind.is_enum():
                return self.xml.find('enumvalue') is not None
            else:
                return False

    class Initializer:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            initializer = self.xml.find('initializer')
            if initializer is not None:
                return self.parser.paras_as_str(initializer, plain=plain)
            else:
                return ''

        def plain(self) -> str:
            return self.md(plain=True)

        def has(self) -> bool:
            return self.xml.find('initializer') is not None

    class Definition:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            return self.plain()

        def plain(self) -> str:
            definition = self.xml.find('definition')
            if definition is not None and definition.text:
                return definition.text + ';'
            else:
                return ''

        def has(self) -> bool:
            return self.xml.find('definition') is not None

    class Programlisting:
        def __init__(self, xml: Element, parser: XmlParser, kind: Kind):
            self.xml = xml
            self.parser = parser
            self.kind = kind

        def md(self, plain: bool = False) -> str:
            programlisting = self.xml.find('programlisting')
            if programlisting is None:
                return ''

            return self.parser.programlisting_as_str(programlisting)

        def has(self) -> bool:
            return self.xml.find('programlisting') is not None



import os
from xml.etree import ElementTree

from doxybook.cache import Cache
from doxybook.constants import Kind, Visibility
from doxybook.node import Node
from doxybook.xml_parser import XmlParser


class Doxygen:
    def __init__(self, index_path: str, parser: XmlParser, cache: Cache, options: dict = {}):
        path = os.path.join(index_path, 'index.xml')
        print('Loading XML from: ' + path)
        xml = ElementTree.parse(path).getroot()

        self.parser = parser
        self.cache = cache
        self._options = options
        self.root = Node('root', None, self.cache, self.parser, None, options=self._options)
        self.groups = Node('root', None, self.cache, self.parser, None, options=self._options)
        self.files = Node('root', None, self.cache, self.parser, None, options=self._options)
        self.pages = Node('root', None, self.cache, self.parser, None, options=self._options)
        self.header_files = Node('root', None, self.cache, self.parser, None, options=self._options)

        for compound in xml.findall('compound'):
            kind = Kind.from_str(compound.get('kind'))
            refid = compound.get('refid')
            node = Node(
                os.path.join(index_path, refid + '.xml'),
                None,
                self.cache,
                self.parser,
                self.root,
                options=self._options,
            )
            node._visibility = Visibility.PUBLIC
            if kind.is_language():
                self.root.add_child(node)
            elif kind == Kind.GROUP:
                self.groups.add_child(node)
            elif kind == Kind.FILE or kind == Kind.DIR:
                self.files.add_child(node)
                if node.is_header_file:
                    self.header_files.add_child(node)
            elif kind == Kind.PAGE:
                self.pages.add_child(node)

        print('Deduplicating data... (may take a minute!)')
        for i, child in enumerate(self.root.children.copy()):
            self._fix_duplicates(child, self.root, [])

        for i, child in enumerate(self.groups.children.copy()):
            self._fix_duplicates(child, self.groups, [Kind.GROUP])

        for i, child in enumerate(self.files.children.copy()):
            self._fix_duplicates(child, self.files, [Kind.FILE, Kind.DIR])

        self._fix_parents(self.files)

        print('Sorting...')
        self._recursive_sort(self.root)
        self._recursive_sort(self.groups)
        self._recursive_sort(self.files)
        self._recursive_sort(self.pages)

    def _fix_parents(self, node: Node):
        if node.is_dir or node.is_root:
            for child in node.children:
                if child.is_file:
                    child._parent = node
                if child.is_dir:
                    self._fix_parents(child)

    def _recursive_sort(self, node: Node):
        node.sort_children()
        for child in node.children:
            self._recursive_sort(child)

    def _is_in_root(self, node: Node, root: Node):
        for child in root.children:
            if node.refid == child.refid:
                return True
        return False

    def _remove_from_root(self, refid: str, root: Node):
        for i, child in enumerate(root.children):
            if child.refid == refid:
                root.children.pop(i)
                return

    def _fix_duplicates(self, node: Node, root: Node, filter: [Kind]):
        for child in node.children:
            if len(filter) > 0 and child.kind not in filter:
                continue
            if self._is_in_root(child, root):
                self._remove_from_root(child.refid, root)
            self._fix_duplicates(child, root, filter)

    def print(self):
        for node in self.root.children:
            self.print_node(node, '')
        for node in self.groups.children:
            self.print_node(node, '')
        for node in self.files.children:
            self.print_node(node, '')

    def print_node(self, node: Node, indent: str):
        print(indent, node.kind, node.name)
        for child in node.children:
            self.print_node(child, indent + '  ')



TEMPLATE = """
# Modules

Here is a list of all modules:

{% for node in nodes recursive %}
{%- if node.is_group %}
* [**{{node.title}}**]({{node.url}}) {{node.brief}}
{%- if node.has_children %}
  {{- loop(node.children)|indent(2, true) }}
{%- endif %}
{%- endif -%}
{%- endfor %}
"""



TEMPLATE = """
# {{node.kind.value|title}} {{node.name_long}}

[**File List**]({{link_prefix}}files.md)
{%- for child in node.parents -%}
{{'**>**'|indent(1, true)}} [**{{child.name_long if node.is_group else child.name_short}}**]({{child.url}})
{%- endfor %}

[Go to the documentation of this file.]({{node.url}})

{{node.programlisting}}
"""



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['doxybook', 'doxybook.templates_python']

package_data = \
{'': ['*'], 'doxybook': ['templates/*', 'templates/c/*', 'templates/cpp/*']}

install_requires = \
['Jinja2']

extras_require = \
{'doc': ['mkdocs', 'mkdocs-material']}

entry_points = \
{'console_scripts': ['esp-doxybook = doxybook.__main__:main']}

setup(name='esp-doxybook',
      version='0.1.0',
      description="Convert Doxygen XML to a single markdown file for C/C++'",
      author=None,
      author_email='Fu Hanxi <fuhanxi@espressif.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )



import os
import time

from jinja2 import select_autoescape, Environment, PackageLoader, FileSystemLoader

from doxybook.cache import Cache
from doxybook.constants import Kind
from doxybook.doxygen import Doxygen
from doxybook.generator import Generator
from doxybook.utils import get_git_revision_hash
from doxybook.xml_parser import XmlParser

import typing as t


def run(
    output: str,
    input_dir: str,
    target: str = 'gitbook',
    hints: bool = True,
    debug: bool = False,
    ignore_errors: bool = False,
    summary: str = None,
    link_prefix: str = '',
    template_dir: t.Optional[str] = None,
    template_lang: t.Optional[str] = 'c',
):
    if output.endswith('.md'):
        os.makedirs(os.path.dirname(output), exist_ok=True)
        output_filepath = output
    else:
        os.makedirs(output, exist_ok=True)
        output_filepath = os.path.join(output, 'api.md')

    options = {'target': target, 'link_prefix': link_prefix}

    cache = Cache()
    parser = XmlParser(cache=cache, target=target, hints=hints)
    doxygen = Doxygen(input_dir, parser, cache, options=options)

    if debug:
        doxygen.print()

    generator = Generator(ignore_errors=ignore_errors, options=options)

    if target == 'single-markdown':
        if template_dir:
            loader = FileSystemLoader(template_dir)
        else:
            loader = PackageLoader('doxybook')
        template_lang = template_lang or 'c'

        env = Environment(loader=loader, autoescape=select_autoescape())
        with open(output_filepath, 'w') as fw:
            template = env.get_template('api.jinja')
            files = doxygen.header_files.children
            common_args = {
                'files': files,
                'file_template': env.get_template(f'{template_lang}/file.jinja'),
                'table_template': env.get_template('table.jinja'),
                'detail_template': env.get_template('detail.jinja'),
                'commit_sha': get_git_revision_hash(),
                'asctime': time.asctime(),
            }
            fw.write(template.render(**common_args))

        print(f'Generated single-markdown API reference: {output_filepath}')
        return

    generator.annotated(output, doxygen.root.children)
    generator.fileindex(output, doxygen.files.children)
    generator.members(output, doxygen.root.children)
    generator.members(output, doxygen.groups.children)
    generator.files(output, doxygen.files.children)
    generator.namespaces(output, doxygen.root.children)
    generator.classes(output, doxygen.root.children)
    generator.hierarchy(output, doxygen.root.children)
    generator.modules(output, doxygen.groups.children)
    generator.pages(output, doxygen.pages.children)
    generator.relatedpages(output, doxygen.pages.children)
    generator.index(
        output,
        doxygen.root.children,
        [Kind.FUNCTION, Kind.VARIABLE, Kind.TYPEDEF, Kind.ENUM],
        [Kind.CLASS, Kind.STRUCT, Kind.INTERFACE],
        'Class Members',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.FUNCTION],
        [Kind.CLASS, Kind.STRUCT, Kind.INTERFACE],
        'Class Member Functions',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.VARIABLE],
        [Kind.CLASS, Kind.STRUCT, Kind.INTERFACE],
        'Class Member Variables',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.TYPEDEF],
        [Kind.CLASS, Kind.STRUCT, Kind.INTERFACE],
        'Class Member Typedefs',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.ENUM],
        [Kind.CLASS, Kind.STRUCT, Kind.INTERFACE],
        'Class Member Enums',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.FUNCTION, Kind.VARIABLE, Kind.TYPEDEF, Kind.ENUM],
        [Kind.NAMESPACE],
        'Namespace Members',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.FUNCTION],
        [Kind.NAMESPACE],
        'Namespace Member Functions',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.VARIABLE],
        [Kind.NAMESPACE],
        'Namespace Member Variables',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.TYPEDEF],
        [Kind.NAMESPACE],
        'Namespace Member Typedefs',
    )
    generator.index(
        output,
        doxygen.root.children,
        [Kind.ENUM],
        [Kind.NAMESPACE],
        'Namespace Member Enums',
    )
    generator.index(output, doxygen.files.children, [Kind.FUNCTION], [Kind.FILE], 'Functions')
    generator.index(output, doxygen.files.children, [Kind.DEFINE], [Kind.FILE], 'Macros')
    generator.index(
        output,
        doxygen.files.children,
        [Kind.VARIABLE, Kind.UNION, Kind.TYPEDEF, Kind.ENUM],
        [Kind.FILE],
        'Variables',
    )

    if target == 'gitbook' and summary:
        generator.summary(
            output,
            summary,
            doxygen.root.children,
            doxygen.groups.children,
            doxygen.files.children,
            doxygen.pages.children,
        )
